/*
 * Copyright (c) 2022, Umut İnan Erdoğan <umutinanerdogan@pm.me>
 *
 * SPDX-License-Identifier: MPL-2.0
 */

pub mod class;
pub mod device;
pub mod vendors;

// MassStorage_IDE (0x0101)
pub mod piix;

pub use class::*;
pub use device::*;

/// Enumerate PCI devices and run initialisation routines on ones we support
pub fn init() {
    for bus in 0..=255 {
        for device in 0..32 {
            if let Some(device_info) = device::check_device(bus, device) {
                if !device::check_pci_support(device_info.device_id) {
                    trace!("PCI device {} on bus {} unsupported", device_info.device, device_info.bus)
                }
            }
        }
    }
}
